==> form-builder-vue-0.7.5.tgz: package/FormWidget/index.ts <==
import { deepAssign } from '@codeperate/utils';
import { ComputedRef, Ref, WatchStopHandle, computed, inject, onBeforeMount, onUnmounted, ref, toRaw, watch } from 'vue';
import { ExportValueOptions, FormBuilderContext, FormSchema, SetValueOptions, ValidatedMeta } from '../FormBuilder';
import { useConfig } from '../config';

export interface FormWidgetProps {
    path: string;
    targetPath?: string;
    required?: boolean;
    validate?: boolean;
    hidden?: boolean;
}
export interface IFormWidget<V = any> {
    isValidated: Ref<boolean>;
    form: FormBuilderContext;
    value: ComputedRef<V>;
    validatedMeta: Ref<ValidatedMeta>;
    view: Ref<boolean>;
    schema: ComputedRef<FormSchema>;
    el?: Ref<HTMLElement | undefined>;
    isRequired: () => boolean;
    isValid: () => boolean | undefined;
    isHidden: () => boolean;
    getSchema: () => FormSchema;
    getPath: () => string;
    getPathArr: () => string[];
    getValue: () => V;
    setValue: (value: any, options?: SetValueOptions) => void;
    register: (exposed?: Partial<IFormWidget<V>> & { [key: string]: any }) => IFormWidget;
    validator: () => void;
    validate: () => ValidatedMeta;
    undoValidate: () => void;
    onExportValue: (wrappedValue: { value: any }, options: ExportValueOptions) => Promise<any>;
    onSaveHistory: (value: any) => Promise<void>;
    onLoadHistory: (value: any) => Promise<void>;
}
export function useFormWidget<V = any, N extends string = any>(name: N, props: FormWidgetProps): Omit<IFormWidget<V>, 'el'> {
    let widget: IFormWidget;
    const isValidated = ref<boolean>(false);
    const validatedMeta = ref<ValidatedMeta>({ validity: true, path: props.path });
    const form = inject<FormBuilderContext>('formBuilder')!;
    const isHidden = () => {
        const hidden = props.hidden || schema.value.hidden;
        if (hidden) return typeof hidden == 'function' ? hidden(widget) : hidden;
        return false;
    };
    const getSchema = () => form.getSchema(props.path);
    const getValue = () => form.getValue(getPath());
    const getPath = () => props.targetPath ?? getSchema().targetPath ?? props.path;
    const getPathArr = () =>
        getPath()
            .split('.')
            .filter(s => s);
    const setValue: IFormWidget['setValue'] = (...args) => form.setValue(getPath(), ...args);

    const isRequired = () => {
        const required = props.required || getSchema().required;
        if (required == null) return false;
        return typeof required == 'function' ? required(widget) : required;
    };
    const isValid = () => {
        if (isValidated.value == true) return validatedMeta.value.validity;
    };
    const value = computed(() => getValue());
    const schema = computed(() => getSchema());
    const config = computed(() => deepAssign(useConfig(name), schema?.value.config ?? {}));
    const register: IFormWidget['register'] = _exposed => {
        widget = { ...exposed, ..._exposed };
        onBeforeMount(() => form!.regWidget(props.path, widget));
        onUnmounted(() => form!.unRegWidget(props.path, widget));

        let watchers: WatchStopHandle[] = [];
        watch(
            () => schema.value.watch,
            watchFn => {
                watchers.forEach(w => w());
                if (!watchFn) return;
                watchers = watchFn.map(({ path, handler }) =>
                    watch(
                        () => form.getValue(typeof path == 'function' ? path(widget) : path),
                        value => handler(value, widget),
                        { immediate: true },
                    ),
                );
            },
            { immediate: true },
        );
        return widget;
    };
    const validator = () => {
        validatedMeta.value.validity = true;
        validatedMeta.value.err = undefined;
    };
    const view = computed(() => schema.value.view ?? form.getView());
    const validate = () => {
        let result = toRaw(validatedMeta.value);
        const schema = getSchema();
        let { validate } = schema;
        validate = typeof validate == 'function' ? validate(widget) : validate;
        if (!view.value && (validate ?? true)) {
            if (schema.validateFn) schema.validateFn(value.value, widget);
            else widget.validator();
            isValidated.value = true;
        }
        return result;
    };
    const undoValidate = () => {
        isValidated.value = false;
        validatedMeta.value.validity = false;
    };
    if (getSchema().defaultValue !== undefined && value.value == undefined)
        setValue(getSchema().defaultValue as V, { disableSideEffect: true });
    onBeforeMount(() => {
        if (widget == undefined) throw new Error('You must call register() function');
    });
    // onRenderTriggered(({ key, target, type }) => {
    //     console.log(name);
    //     console.log(`Render triggered on component`, { key, target, type });
    // });

    const onExportValue = async () => {};
    const onSaveHistory = async () => {};
    const onLoadHistory = async () => {};
    const exposed = {
        isValidated,
        form,
        value,
        validatedMeta,
        view,
        schema,
        config,
        isHidden,
        isValid,
        isRequired,
        getSchema,
        getPath,
        getPathArr,
        setValue,
        getValue,
        register,
        validator,
        validate,
        undoValidate,
        onExportValue,
        onSaveHistory,
        onLoadHistory,
    };
    return exposed;
}

==> form-builder-vue-0.7.9.tgz: package/FormWidget/index.ts <==
import { deepAssign } from '@codeperate/utils';
import { ComputedRef, Ref, WatchStopHandle, computed, inject, onBeforeMount, onUnmounted, ref, toRaw, watch } from 'vue';
import { ExportValueOptions, FormBuilderContext, FormSchema, SetValueOptions, ValidatedMeta } from '../FormBuilder';
import { useConfig } from '../config';

export interface FormWidgetProps {
    path: string;
    targetPath?: string;
    required?: boolean;
    validate?: boolean;
    hidden?: boolean;
}
export interface IFormWidget<V = any> {
    isValidated: Ref<boolean>;
    form: FormBuilderContext;
    value: ComputedRef<V>;
    validatedMeta: Ref<ValidatedMeta>;
    view: Ref<boolean>;
    schema: ComputedRef<FormSchema>;
    el?: Ref<HTMLElement | undefined>;
    isRequired: () => boolean;
    isValid: () => boolean | undefined;
    isHidden: () => boolean;
    getSchema: () => FormSchema;
    getPath: () => string;
    getPathArr: () => string[];
    getValue: () => V;
    setValue: (value: any, options?: SetValueOptions) => void;
    register: (exposed?: Partial<IFormWidget<V>> & { [key: string]: any }) => IFormWidget;
    validator: () => void;
    validate: () => ValidatedMeta;
    undoValidate: () => void;
    onExportValue: (wrappedValue: { value: any }, options: ExportValueOptions) => Promise<any>;
    onSaveHistory: (wrappedValue: { value: any }) => Promise<void>;
    onLoadHistory: (wrappedValue: { value: any }) => Promise<void>;
}
export function useFormWidget<V = any, N extends string = any>(name: N, props: FormWidgetProps): Omit<IFormWidget<V>, 'el'> {
    let widget: IFormWidget;
    const isValidated = ref<boolean>(false);
    const validatedMeta = ref<ValidatedMeta>({ validity: true, path: props.path });
    const form = inject<FormBuilderContext>('formBuilder')!;
    const isHidden = () => {
        const hidden = props.hidden || schema.value.hidden;
        if (hidden) return typeof hidden == 'function' ? hidden(widget) : hidden;
        return false;
    };
    const getSchema = () => form.getSchema(props.path);
    const getValue = () => form.getValue(getPath());
    const getPath = () => props.targetPath ?? getSchema().targetPath ?? props.path;
    const getPathArr = () =>
        getPath()
            .split('.')
            .filter(s => s);
    const setValue: IFormWidget['setValue'] = (...args) => form.setValue(getPath(), ...args);

    const isRequired = () => {
        const required = props.required || getSchema().required;
        if (required == null) return false;
        return typeof required == 'function' ? required(widget) : required;
    };
    const isValid = () => {
        if (isValidated.value == true) return validatedMeta.value.validity;
    };
    const value = computed(() => getValue());
    const schema = computed(() => getSchema());
    const config = computed(() => deepAssign(useConfig(name), schema?.value.config ?? {}));
    const register: IFormWidget['register'] = _exposed => {
        widget = { ...exposed, ..._exposed };
        onBeforeMount(() => form!.regWidget(props.path, widget));
        onUnmounted(() => form!.unRegWidget(props.path, widget));

        let watchers: WatchStopHandle[] = [];
        watch(
            () => schema.value.watch,
            watchFn => {
                watchers.forEach(w => w());
                if (!watchFn) return;
                watchers = watchFn.map(({ path, handler }) =>
                    watch(
                        () => form.getValue(typeof path == 'function' ? path(widget) : path),
                        value => handler(value, widget),
                        { immediate: true },
                    ),
                );
            },
            { immediate: true },
        );
        return widget;
    };
    const validator = () => {
        validatedMeta.value.validity = true;
        validatedMeta.value.err = undefined;
    };
    const view = computed(() => schema.value.view ?? form.getView());
    const validate = () => {
        let result = toRaw(validatedMeta.value);
        const schema = getSchema();
        let { validate } = schema;
        validate = typeof validate == 'function' ? validate(widget) : validate;
        if (!view.value && (validate ?? true)) {
            if (schema.validateFn) schema.validateFn(value.value, widget);
            else widget.validator();
            isValidated.value = true;
        }
        return result;
    };
    const undoValidate = () => {
        isValidated.value = false;
        validatedMeta.value.validity = false;
    };
    if (getSchema().defaultValue !== undefined && value.value == undefined)
        setValue(getSchema().defaultValue as V, { disableSideEffect: true });
    onBeforeMount(() => {
        if (widget == undefined) throw new Error('You must call register() function');
    });
    // onRenderTriggered(({ key, target, type }) => {
    //     console.log(name);
    //     console.log(`Render triggered on component`, { key, target, type });
    // });

    const onExportValue = async () => {};
    const onSaveHistory = async () => {};
    const onLoadHistory = async () => {};
    const exposed = {
        isValidated,
        form,
        value,
        validatedMeta,
        view,
        schema,
        config,
        isHidden,
        isValid,
        isRequired,
        getSchema,
        getPath,
        getPathArr,
        setValue,
        getValue,
        register,
        validator,
        validate,
        undoValidate,
        onExportValue,
        onSaveHistory,
        onLoadHistory,
    };
    return exposed;
}
